package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.exception.ParamNullException;
import com.zanxiang.common.utils.GameSignUtil;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.Game;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.mapper.UserMapper;
import com.zanxiang.sdk.common.util.RegisterUtil;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UpdatePasswordParam;
import com.zanxiang.sdk.domain.params.UserCheckParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.params.UserDetailParam;
import com.zanxiang.sdk.domain.vo.UserVO;
import com.zanxiang.sdk.service.GameService;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.service.UserTokenService;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户信息
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserTokenService userTokenService;

    @Autowired
    private GameService gameService;

    /**
     * 用户重置密码
     *
     * @param updatePasswordParam : 修改密码的参数
     * @param userData            : userData
     * @return : 返回修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> updatePassword(UpdatePasswordParam updatePasswordParam, UserData userData) {
        String oldPassword = updatePasswordParam.getOldPassword();
        String newPassword = updatePasswordParam.getNewPassword();
        //查询用户信息
        User user = super.getById(userData.getUserId());
        //旧密码校验
        if (Strings.isBlank(user.getPassword()) || !Objects.equals(RegisterUtil.cmfPassword(oldPassword), user.getPassword())) {
            return new ResultVO<>(HttpStatusEnum.OLD_PASSWORD_ERROR);
        }
        //新密码校验
        HttpStatusEnum checkPasswordEnum = RegisterUtil.checkPassword(newPassword);
        if (!Objects.equals(checkPasswordEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVO<>(checkPasswordEnum);
        }
        //修改用户密码
        super.update(new LambdaUpdateWrapper<User>()
                .set(User::getPassword, RegisterUtil.cmfPassword(newPassword))
                .set(User::getUpdateTime, LocalDateTime.now()));
        //返回修改成功
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 获取用户信息
     *
     * @param userDetailParam : 用户详情参数
     * @param userData        : userData
     * @return : 返回用户信息
     */
    @Override
    public ResultVO<UserVO> getUserDetail(UserDetailParam userDetailParam, UserData userData) {
        String token = userDetailParam.getToken();
        Integer deviceType = userData.getDeviceType();
        UserTokenDTO userTokenDTO = userTokenService.getTokenInfoByTokenDevice(token, deviceType);
        if (userTokenDTO == null) {
            return new ResultVO<>(HttpStatusEnum.INVALID_PARAMS);
        }
        //查询用户信息
        User user = super.getById(userTokenDTO.getUserId());
        if (user == null) {
            return new ResultVO<>(HttpStatusEnum.UNKNOWN_ERROR);
        }
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        return new ResultVO<>(HttpStatusEnum.SUCCESS, userVO);
    }

    /**
     * 根据手机号获取用户信息
     *
     * @param mobile : 用户手机号
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getUserInfoByMobile(String mobile) {
        User user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        return BeanUtils.copy(user, UserDTO.class);
    }

    @Override
    public Boolean userCheck(UserCheckParam param, UserData userData) {
        if (StringUtils.isEmpty(param.getSign())) {
            throw new ParamNullException("Sign");
        }
        //查询gameUserId
        if (StringUtils.isEmpty(param.getGameId())) {
            throw new ParamNullException("GameId");
        }
        Game gameData = gameService.getById(param.getGameId());
        if (Objects.isNull(gameData)) {
            throw new CustomException(HttpStatusEnum.GAME_UN_FUND);
        }
        SortedMap<String, String> map = new TreeMap<>();
        map.put("sign", param.getSign());
        map.put("gameId", param.getGameId());
        map.put("userId", String.valueOf(userData.getUserId()));
        return GameSignUtil.verify(map, gameData.getAppKey());
    }

}
